@@ -160,10 +160,7 @@ DNS_RR *dns_rr_create(const char *qname, const char *rname,
 {
     DNS_RR *rr;
 
-    /*
-     * Note: if this function is changed, update dns_rr_copy().
-     */
-    rr = (DNS_RR *) mymalloc(sizeof(*rr));
+    rr = (DNS_RR *) mymalloc(sizeof(*rr) + data_len - 1);
     rr->qname = mystrdup(qname);
     rr->rname = mystrdup(rname);
     rr->type = type;
@@ -173,12 +170,8 @@ DNS_RR *dns_rr_create(const char *qname, const char *rname,
     rr->pref = pref;
     rr->weight = weight;
     rr->port = port;
-    if (data_len != 0) {
-	rr->data = mymalloc(data_len);
+    if (data && data_len > 0)
 	memcpy(rr->data, data, data_len);
-    } else {
-	rr->data = 0;
-    }
     rr->data_len = data_len;
     rr->next = 0;
     return (rr);
@@ -193,8 +186,6 @@ void    dns_rr_free(DNS_RR *rr)
 	    dns_rr_free(rr->next);
 	myfree(rr->qname);
 	myfree(rr->rname);
-	if (rr->data)
-	    myfree(rr->data);
 	myfree((void *) rr);
     }
 }
@@ -203,17 +194,16 @@ void    dns_rr_free(DNS_RR *rr)
 
 DNS_RR *dns_rr_copy(DNS_RR *src)
 {
+    ssize_t len = sizeof(*src) + src->data_len - 1;
     DNS_RR *dst;
 
     /*
-     * Note: struct copy, because dns_rr_create() would not copy all fields.
+     * Combine struct assignment and data copy in one block copy operation.
      */
-    dst = (DNS_RR *) mymalloc(sizeof(*dst));
-    *dst = *src;
+    dst = (DNS_RR *) mymalloc(len);
+    memcpy((void *) dst, (void *) src, len);
     dst->qname = mystrdup(src->qname);
     dst->rname = mystrdup(src->rname);
-    if (dst->data)
-	dst->data = mymemdup(src->data, src->data_len);
     dst->next = 0;
     return (dst);
 }
